﻿using System;
namespace nick3216.Library.Crypt.Password
{
    /// <summary>
    /// Interface to be implemented by language classes used by the 
    /// <see cref="Generator"/> class.
    /// 
    /// <h1>Authors</h1>
    /// <para>Nick Wallis &lt;<a href='mailto:singlespeeder@btinternet.com'>singlespeeder@btinternet.com</a>&gt;<br/>
    /// Based on the perl <a href='http://search.cpan.org/~jwalt/Crypt-GeneratePassword-0.03/'>Crypt::GeneratePassword</a>
    /// module by Jörg Walter.<br/>
    /// Inspired by ideas from Tom Van Vleck and Morris Gasser/FIPS-181.</para>
    /// 
    /// <h1>Copyright</h1>
    /// <para>Copyright (C) 2009, Nick Wallis &lt;<a href='mailto:singlespeeder@btinternet.com'>singlespeeder@btinternet.com</a>&gt;<br/>
    /// Portions Copyright (C) 2002, Jörg Walter &lt;<a href='mailto:jwalt@cpan.org'>jwalt@cpan.org</a>&gt;<br/>
    /// All rights reserved.</para>
    ///
    /// <para>This file is part of nick3216.Library.Crypt.Password.
    /// nick3216.Library.Crypt.Password is free software: you can 
    /// redistribute it and/or modify it under the terms of the GNU General 
    /// Public License as published by the Free Software Foundation, either 
    /// version 3 of the License, or (at your option) any later version.</para>
    ///
    /// <para>nick3216.Library.Crypt.Password is distributed in the hope that it 
    /// will be useful, but WITHOUT ANY WARRANTY; without even the implied 
    /// warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
    /// See the GNU General Public License for more details.</para>
    ///
    /// <para>You should have received a copy of the GNU General Public License
    /// along with nick3216.Library.Crypt.Password.  
    /// If not, see <a href='http://www.gnu.org/licenses/'>http://www.gnu.org/licenses/</a></para>
    /// </summary>
    public interface ILanguage
    {
        /// <summary>
        /// Gets a specified character set or sets.
        /// </summary>
        /// <value>The set or sets.</value>
        String[] CharacterSets
        {
            get;
        }

        /// <summary>
        /// Gets the default character set for this language as a String.
        /// </summary>
        /// <value>The default set.</value>
        String DefaultCharacterSet
        {
            get;
        }

        /// <summary>
        /// Regular expression filter pattern used to determine if character
        /// sequences could be considered offensive.
        /// </summary>
        /// <value>The filter pattern.</value>
        String Filter
        {
            get;
        }

        /// <summary>
        /// Gets the frequency of the specified character sequence in this 
        /// language.
        /// </summary>
        /// <param name="sequence">The character sequence.</param>
        int Frequency(char[] sequence);

        /// <summary>
        /// Gets a character set for this language. Returns the lowercase 
        /// letters by default, but can include symbols (digits and some 
        /// arithmetic symbols) or uppercase letters.
        /// </summary>
        /// <param name="useSymbols">If set to <c>true</c> include symbols 
        /// in the returned character set.</param>
        /// <param name="useCapitals">If set to <c>true</c> include uppercase 
        /// letters  in the returned character set.</param>
        /// <returns>A string containing a character set for this language.</returns>
        String GetCharacterSet(bool useSymbols, bool useCapitals);

        /// <summary>
        /// Gets the max quadgram frequency, i.e. the frequency of the most 
        /// common four character sequence.
        /// </summary>
        /// <value>The max quadgram frequency.</value>
        int MaxQuadgramFrequency
        {
            get;
        }

        /// <summary>
        /// Gets the max trigram frequency, i.e. the frequency of the most 
        /// common three character sequence.
        /// </summary>
        /// <value>The max trigram frequency.</value>
        int MaxTrigramFrequency
        {
            get;
        }

        /// <summary>
        /// Gets the name of the language.
        /// </summary>
        /// <value>The name of the language.</value>
        String Name
        {
            get;
        }
    }
}
